class OmnikassaException(Exception):
    pass


class InvalidResponseCode(OmnikassaException):
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return 'Got responseCode {}'.format(self.data['responseCode'])


class InvalidSeal(OmnikassaException):
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return 'Invalid seal: {}'.format(self.data)



import logging

from hashlib import sha256
from hmac import compare_digest


from .payment import Payment
from .exceptions import (
    OmnikassaException,
    InvalidResponseCode,
    InvalidSeal,
)
from .constants import (
    Settings,
    ResponseCode,
)
from .helpers import str_to_data

log = logging.getLogger(__name__)


class Omnikassa:
    def __init__(self, settings, secret=None):
        if secret:
            self.url = Settings.URL
            self.secret = secret
        else:
            settings['merchantId'] = Settings.Test.MERCHANT_ID
            self.secret = Settings.Test.SECRET_KEY
            self.url = Settings.Test.URL
        self.settings = self._validate_settings(settings)

    def _validate_settings(self, settings):
        if not settings.get('merchantId'):
            raise OmnikassaException('merchantId is required.')
        if not settings.get('normalReturnUrl'):
            raise OmnikassaException('normalReturnUrl is required.')
        if not settings.get('automaticResponseUrl'):
            raise OmnikassaException('automaticResponseUrl is required.')
        if not settings.get('keyVersion'):
            settings['keyVersion'] = '1'
        return settings

    def generate_request(self, settings):
        payment = self.settings.copy()
        payment.update(settings)
        payment = Payment(payment)
        return {
            'url': self.url,
            'data': payment,
            'seal': self._generate_seal(payment),
            'version': Settings.VERSION,
        }

    def verify_callback(self, data):
        if not self._check_seal(data['Seal'], data['Data']):
            raise InvalidSeal(data)
        data = str_to_data(data['Data'])
        if data['responseCode'] != ResponseCode.SUCCESS:
            raise InvalidResponseCode(data)
        return data

    def _generate_seal(self, data):
        data = str(data) + self.secret
        return sha256(data.encode()).hexdigest()

    def _check_seal(self, seal, data):
        return compare_digest(seal, self._generate_seal(data))



class Settings:
    class Test:
        MERCHANT_ID = '002020000000001'
        SECRET_KEY = MERCHANT_ID + '_KEY1'
        URL = 'https://payment-webinit.simu.omnikassa.rabobank.nl/paymentServlet'
    URL = 'https://payment-webinit.omnikassa.rabobank.nl/paymentServlet'
    VERSION = 'HP_1.0'


class Currency:
    EURO = '978'


class ResponseCode:
    SUCCESS = '00'



def data_to_str(data):
    values = []
    for key, value in data.items():
        values.append('{}={}'.format(key, value))
    return '|'.join(values)


def str_to_data(string):
    pairs = string.split('|')
    data = {}
    for p in pairs:
        key, value = p.split('=', 1)
        data[key] = value
    return data




import os

from setuptools import (
    setup,
    find_packages,
)

HERE = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(HERE, 'README.rst')).read()
CHANGES = open(os.path.join(HERE, 'CHANGES.rst')).read()

requires = [
]

setup(
    name='omnikassa',
    version='0.0.4',
    description='Python library for implementing Omnikassa',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Programming Language :: Python",
    ],
    author='Rob Wouters',
    author_email='rob@rob-wouters.nl',
    url='https://github.com/RobWouters/omnikassa',
    download_url='https://github.com/RobWouters/omnikassa/tarball/0.0.4',
    keywords='python omnikassa payment',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
)



import time

from .constants import Settings

from .exceptions import OmnikassaException
from .helpers import data_to_str


class Payment:
    def __init__(self, settings):
        self.settings = self._validate_settings(settings.copy())

    def _validate_settings(self, settings):
        if not settings.get('currencyCode'):
            raise OmnikassaException('currencyCode is required.')

        if not settings.get('transactionReference'):
            raise OmnikassaException('transactionReference is required.')

        if not settings.get('amount'):
            raise OmnikassaException('amount is required.')
        if type(settings['amount']) != int:
            raise OmnikassaException('amount should be an integer (cents).')

        if not settings.get('normalReturnUrl'):
            raise OmnikassaException('normalReturnUrl is required.')

        if settings['merchantId'] == Settings.Test.MERCHANT_ID:
            # Prefix reference in test with timestamp to prevent collisions.
            settings['transactionReference'] = '{}{}'.format(
                time.time(), settings['transactionReference']
            ).replace('.', '')

        return settings

    def __str__(self):
        return data_to_str(self.settings)
